
# ==========================================================
# Author : Mikomi
# ==========================================================
# Version : 0.1
# Create : 2007/05/10 
# Last version : 2007/05/14 
# ==========================================================
# Config Manager
# ==========================================================
# Description :
# Manage the config part of the application
# ==========================================================

puts "require ConfigManager"

# ----------------------------------------------------------
require "Common"
require "constants/CstsConfigManager"
require "constants/CstsKaraoke"
require "helper/Property"
# ----------------------------------------------------------

puts "++ ConfigManager"

module Application
  
  class ConfigManager
    include Common
    include CstsConfigManager
    
    LIST_SEPARATOR = ";"
    
    # default column order
    DEFAULT_COLUMN_ORDER = [ 
      CstsKaraoke::K_ID,
      CstsKaraoke::K_LENGTH,
      CstsKaraoke::K_USED,
      CstsKaraoke::K_LAST,
      CstsKaraoke::K_FULL_TITLE,
      CstsKaraoke::K_INI,
      CstsKaraoke::K_VIDEO,
      CstsKaraoke::K_LYRICS,
      CstsKaraoke::K_FPS,
      CstsKaraoke::K_FREQ,
      CstsKaraoke::K_WIDTH,
      CstsKaraoke::K_HEIGHT,
      CstsKaraoke::K_FOURCC,
      CstsKaraoke::K_VIDEO_CODEC,
      CstsKaraoke::K_AUDIO_CODEC,
    ]
    
    # default column and visibility
    DEFAULT_COLUMN = {
      CstsKaraoke::K_ID => true,
      CstsKaraoke::K_LENGTH => true,
      CstsKaraoke::K_USED => true,
      CstsKaraoke::K_LAST => true,
      CstsKaraoke::K_FULL_TITLE => true,
      CstsKaraoke::K_INI => false,
      CstsKaraoke::K_VIDEO => false,
      CstsKaraoke::K_LYRICS => false,
      CstsKaraoke::K_FPS => false,
      CstsKaraoke::K_FREQ => false,
      CstsKaraoke::K_WIDTH => false,
      CstsKaraoke::K_HEIGHT => false,
      CstsKaraoke::K_FOURCC => false,
      CstsKaraoke::K_VIDEO_CODEC => false,
      CstsKaraoke::K_AUDIO_CODEC => false,
    }
    
    # =================================
    public
    # ---------------------------------
    def initialize(pProfile)
      @profile = pProfile
      loadConfig()
      configure()
    end
    
    # ---------------------------------
    def save()
      log("save") {
        prepareSave()
        saveConfig()
      }
    end

    # ---------------------------------
    # return the config value for pId
    def [](pId)
      return @map[pId]
    end
    
    # ---------------------------------
    # set the config value for pId
    def []=(pId, pValue)
      # return @map[pId] = pValue.to_i if isNumber(pValue)
      # return @map[pId] = (true.to_s == pValue.to_s) if isBoolean(pValue)
      return if pValue.nil?
      log("Config : "+pId+" = "+pValue.to_s)
      @map[pId] = pValue.to_s
    end


    # ---------------------------------
    
    # if we can delete this column
    def isColumnDeletable(pColumnKey)
      return !DEFAULT_COLUMN.include?(pColumnKey)
    end
    
    # return the column order
    def getColumnOrder()
      return @columnOrder if @columnOrder
      columnLine = @map[COLUMN_ORDER]
      if columnLine
        @columnOrder = columnLine.split(LIST_SEPARATOR)
      else
        @columnOrder = Array.new
      end
      updateColumnOrder()
      return @columnOrder
    end
    
    # set the column order
    def setColumnOrder(pOrder)
      @columnOrder = pOrder
    end
    
    # return the column displayed in the respective order
    def getColumnDisplay()
      return @columnDisplay if @columnDisplay
      columnLine = @map[COLUMN_DISPLAY]
      if columnLine
        @columnDisplay = columnLine.split(LIST_SEPARATOR)
      else
        @columnDisplay = Array.new
      end
      updateColumnDisplay()
      return @columnDisplay
    end
    
    # set the column display order
    def setColumnDisplay(pOrder)
      @columnDisplay = pOrder
    end

    # ---------------------------------
    def getIniFilePath()
      return @map[INI_FILE_PATH]
    end

    # ---------------------------------
    def getVideoFilePath()
      return @map[VIDEO_FILE_PATH]
    end

    # ---------------------------------
    def getLyricsFilePath()
      return @map[LYRICS_FILE_PATH]
    end

    # ---------------------------------
    def getPlaylistFileName()
      return @map[PLAYLIST_FILE_NAME]
    end
    
    # ---------------------------------
    def getBackupFilePath()
      return @map[BACKUP_FILE_PATH]
    end

    # ---------------------------------
    def getBackupFileName()
    	currentTime = Time.new
			dateString = currentTime.strftime("%Y-%m-%d--%H-%M-%S")
    	filepath = getBackupFilePath()
    	filename = PLAYLIST_BACKUP+'_'+@profile+'_'+dateString+PLAYLIST_EXT
      return File.join(filepath, filename)
    end
    
    # ---------------------------------
    def getResumeFileName()
    	currentTime = Time.new
			dateString = currentTime.strftime("%Y-%m-%d--%H-%M-%S")
    	filepath = getBackupFilePath()
    	filename = PLAYLIST_RESUME+'_'+@profile+'_'+dateString+PLAYLIST_EXT
      return File.join(filepath, filename)
    end
    
    # ---------------------------------
    def getDataFileName()
      return File.join(CONFIG_PATH, @profile, @profile+DATA_SUFFIXE)
    end

    # ---------------------------------
    def getColumnFileName()
      return File.join(CONFIG_PATH, @profile, @profile+COLUMN_SUFFIXE)
    end

    # ---------------------------------
    def getCurrentPlaylistFileName()
      return File.join(CONFIG_PATH, @profile, @profile+CURRENT_PLAYLIST_SUFFIXE)
    end
    
    # ---------------------------------
    def getFilterFileName()
      return File.join(CONFIG_PATH, @profile, @profile+FILTER_SUFFIXE)
    end
    
    # ---------------------------------
    def isShuffleOnGenerate()      
      return is(SHUFFLE_ON_GENERATE)
    end

    def isLaunchOnGenerate()
      return is(LAUNCH_ON_GENERATE)
    end

    # return the boolean value for the given key
    # return false if nil
    def is(pKey)
      bool = @map[pKey]
      return false if bool.nil?
      return (true.to_s == bool.to_s)
    end

    # return the number value for the given key, 0 if error
    def number(pKey)
      num = @map[pKey]
      return 0 if num.nil?
      begin
        return num.to_i
      rescue
        return 0
      end
    end


    # =================================
    private
    # ---------------------------------
    def getConfigFileName()
      return File.join(CONFIG_PATH, @profile, @profile+CONFIG_SUFFIXE)
    end

    # ---------------------------------
    def loadConfig()
      log("loading profile "+getConfigFileName())
      @map = Property::load(getConfigFileName())
      unless @map
        log("config file "+getConfigFileName()+" not found")
        @map = Hash.new
      end
    end
    
    # ---------------------------------
    def saveConfig()
      log("saving profile "+getConfigFileName())
      Property::save(getConfigFileName(), @map)
    end
    
    # prepare data before saving
    def prepareSave()
      @map[COLUMN_ORDER] = getColumnOrder().join(LIST_SEPARATOR)
      @map[COLUMN_DISPLAY] = getColumnDisplay().join(LIST_SEPARATOR)
    end

    # configure data after loading
    def configure()
      getColumnOrder()
      getColumnDisplay()
    end
    
    # update the column order adding obligatory column
    def updateColumnOrder()
      DEFAULT_COLUMN_ORDER.each { |bColumn|
        @columnOrder << bColumn unless @columnOrder.include?(bColumn)
      }
      
    end

    # update column display adding obligatory column (invisible)
    def updateColumnDisplay()
        falseStr = false.to_s
        while @columnOrder.size > @columnDisplay.size
          column = @columnOrder[@columnDisplay.size() - 1]
          if DEFAULT_COLUMN.include?(column)
            @columnDisplay << DEFAULT_COLUMN[column].to_s 
          else
            @columnDisplay << falseStr
          end
        end
    end

    # test if the given value is a number
    def isNumber(pValue)
    	begin
      	return pValue.to_s === pValue.to_i.to_s
      rescue Exception => pException
      	return false
    	end
    end
    
    # test if the given value is a boolean
    def isBoolean(pValue)
      return ((true.to_s == pValue.to_s) || false.to_s == pValue.to_s)
    end
    
  end
end

puts "-- ConfigManager"

