
# ==========================================================
# Author : Mikomi
# ==========================================================
# Version : 0.1
# Create : 2007/05/10 
# Last version : 2007/05/11 
# ==========================================================
# Column Selector GUI
# ==========================================================
# Description :
# Sub module for the Toyunda Manager GUI. Only contains
# callback for the Column selector (include in the ConfigManagerGUI)
# Uses methodes and attributs from the main class : 
# ToyundaManagerGUI
# ==========================================================

puts "require ColumnSelectorGUI"

# ----------------------------------------------------------
require "gui/GUI"
require "gui/SubToyundaManagerGUI"
require "constants/CstsKaraoke"
require 'gui/ColumnMap'

# ----------------------------------------------------------

puts "++ ColumnSelectorGUI"

module GUI

  class ColumnSelectorGUI < SubToyundaManagerGUI
    
    W_VIEW_COLUMN_SELECTOR = 'treeviewColumnSelector'
    W_ENTRY_NEW_COLUMN = 'entryNewColumn'
    W_BUTTON_ADD = 'buttonAddColumn'
    W_BUTTON_DELETE = 'buttonDeleteColumn'
    W_CHECK_EDITABLE = 'checkEditable'
    
    attr_reader :sortColumnSelector
    
    # =====================================
    # Initialize
    # =====================================
    
    # clear the config manager
    def clear()
      getColumnSelectorView().columns.each { |bColumn|
        getColumnSelectorView().remove_column(bColumn)
      }
      @columnSelectorModel = nil
      @selectTitleColumns = nil
    end
    
    # initialize the config manager
    def subInitialize()
      log("subInitialize") {
        @changed = true
        @sortColumnSelector = [CstsKaraoke::K_FULL_TITLE]
        iniFunctions()
        iniColumnSelectorView()
      }
    end

    # =====================================
    public
    # =====================================

    # callback for saving information about ColumnSelectorGUI
    def save()
      log("save"){
        # save column order and display
        colOrder = Array.new()
        colDisplay = Array.new()
        
        # !!! TODO !!! The column list may not be up to date !!!
        getSelectTitleColumns().each { |bColumnMap|
          colOrder << bColumnMap.title
          colDisplay << bColumnMap.visible?.to_s
        }
        @application.configManager.setColumnOrder(colOrder)
        @application.configManager.setColumnDisplay(colDisplay)
        log("columnOrder = "+colOrder.join(", "))
        log("columnDisplay = "+colDisplay.join(", "))
        
        #~ # save the model into a file
        #~ fileName = @application.configManager.getColumnFileName()
        #~ File.open(fileName,'w') { |bFile|
          #~ Marshal.dump(getColumnSelectorModel(), bFile)
        #~ }
      }
    end

    # return an array of columns for the select title view
    def getSelectTitleColumns()
      log("getSelectTitleColumns") {
        return @selectTitleColumns if @selectTitleColumns
        iniSelectTitleColumns()
        return @selectTitleColumns
      }
    end
  
    # return the treeview for the column selector
    def getColumnSelectorView()
      return @glade[W_VIEW_COLUMN_SELECTOR]
    end

    # return the treeview for the column selector
    def getColumnSelectorModel()
      return @columnSelectorModel if @columnSelectorModel
      iniColumnSelectorModel()
      return @columnSelectorModel
    end

    # if the columns displayed have changed
    def changed?()
      return @changed
    end

    # set the changed status
    def changed(pValue = true)
      @changed = pValue
      @selectTitleColumns = nil
      @toyundaManagerGUI.getPlaylistManagerGUI().updateSelectTitleColumns()
    end

    # update column from karaoke list
    def updateColumnsFromKaraokeList(pKaraokeList)
      map = Hash.new
      getSelectTitleColumns().each { |bColumnMap|
        map[bColumnMap.title] = true
      }
      newList = Array.new
      pKaraokeList.each { |bKaraoke|
        bKaraoke.each { |bKey|
          unless map.key?(bKey)
            map[bKey] = true
            newList << bKey
          end
        }
      }
      newList.each { |bKey|
        addColumn(getColumnSelectorModel(),bKey)
      }
    end

    # =====================================
    private
    # =====================================

    # initialize various functions
    def iniFunctions()
      # select function
      @selectedFunc = proc {	|bKaraoke|
      	karaoke = @toyundaManagerGUI.getPlaylistManagerGUI().currentPlaylist.getById(bKaraoke.id)
      	!karaoke.nil?
      }
      
      # clicked function
      @clickedFunc = proc { |bColumnMap|
        @sortColumnSelector.delete(bColumnMap.key)
        @sortColumnSelector.insert(0, bColumnMap.key)
        log("sort : "+@sortColumnSelector.join(", "))
      }
      
      # edit function
      @editFunc = proc { |bColumnMap, bPath, bValue|
        model = @toyundaManagerGUI.getPlaylistManagerGUI().getModelSelector()
        row = model.get_iter(bPath)
        karaoke = row[0]
        bValue = nil	if bValue=~/^\s*$/
        karaoke[bColumnMap.key] = bValue
        log("New value : "+karaoke.to_s+"["+bColumnMap.key+"] = "+karaoke[bColumnMap.key].to_s+" ("+karaoke[bColumnMap.key].class.to_s+")")
      }
    end

    # initialize the view for the column selector
    def iniColumnSelectorView()
      log("iniColumnSelectorView") {
        view = getColumnSelectorView()
        view.model = getColumnSelectorModel()
        view.selection.mode = Gtk::SELECTION_BROWSE
        view.enable_search = false
        
        renderer = Gtk::CellRendererText.new

        gtkColumn = Gtk::TreeViewColumn.new("Name",renderer)
        gtkColumn.resizable = true
        gtkColumn.clickable = false
        gtkColumn.set_cell_data_func(renderer) { |bColumn, bRenderer, bModel, bRow|
          columnMap = bRow[0]
          if columnMap.visible?
            bRenderer.background_gdk = columnMap.style.background
          else
            bRenderer.background_gdk = ColumnMap._darker(columnMap.style.background, columnMap.style.darkerValue)
          end
          bRenderer.foreground_gdk = columnMap.style.foreground
          bRenderer.font = columnMap.style.font
          bRenderer.text = columnMap.to_s
        }

        view.append_column(gtkColumn)
      }
    end

    # return the model of ColumnMap for the select title view
    def iniColumnSelectorModel()
      log("iniColumnSelectorModel") {
        #~ @columnSelectorModel = createColumnSelectorModelFromFile()
        #~ return unless @columnSelectorModel.nil?
        @columnSelectorModel = createColumnSelectorModelFromConfig()
      }
    end
    
    # load model from config file
    def createColumnSelectorModelFromConfig()
        # create from config
        model = Gtk::ListStore.new(ColumnMap, String)
        i = 0
        colDisplay = @application.configManager.getColumnDisplay()
        @application.configManager.getColumnOrder().each { |bColumnName|
          column = createColumnMap(bColumnName)
          column.visible = (true.to_s == colDisplay[i])
          addColumn(model, column)
          i += 1
        }
        return model
    end

    # create the model from the column file. return nil if no file
    def createColumnSelectorModelFromFile()
        fileName = @application.configManager.getColumnFileName()
        return nil unless File.exist?(fileName)
        model = nil
        begin
          File.open(fileName,'r') { |bFile|
            model = Marshal.load(bFile)
          }
        rescue
        end
        return model
    end

    # return the list of ColumnMap for the select title view
    def iniSelectTitleColumns()
      log("iniSelectTitleColumns") {
        @selectTitleColumns = Array.new
        getColumnSelectorModel().each { |bModel,bPath,bIter|
          columnMap = bIter[0]
          @selectTitleColumns << columnMap
        }
      }
    end

    # add the given column in the given model
    def addColumn(pModel, pColumn)
      row = pModel.append()
      row[0] = pColumn
      row[1] = pColumn.to_s
    end

    # create and configure a column
    def createColumnMap(pColumnName)
      column = ColumnMap.new(pColumnName)
      column.setSelectedFunc(@selectedFunc)
      column.setClickedFunc(@clickedFunc)
      column.setEditFunc(@editFunc)
      return column
    end

    # =====================================
    # Callbacks
    # =====================================
    public
    
    # when activating a row : show / hide column
    def on_treeviewColumnSelector_row_activated(pWidget, pPath, pColumn)
      log("toggle visibility") {
        rowSelected = pWidget.model.get_iter(pPath)
        columnMap = rowSelected[0]
        columnMap.visible = !columnMap.visible?
        log(columnMap.to_s+" : set visible = "+columnMap.visible?.to_s)
        changed()
      }
    end
    
    # select a row in the column selector view : display column info
    def on_treeviewColumnSelector_cursor_changed(pWidget)
      selectedRow = getColumnSelectorView().selection.selected
      columnMap = selectedRow[0]
      log("column info : "+columnMap.to_s)
      # column editable ?
      @glade[W_CHECK_EDITABLE].active = columnMap.editable?
    end
    
    # when moving row in the selector
    def on_treeviewColumnSelector_drag_end(pWidget, pArg1=nil, pArg2=nil, pArg3=nil)
      log("dragged & dropped") {
        changed()
      }
    end

    # clic on Add Column button
    def on_buttonAddColumn_clicked(pWidget)
      log("add column") {
        name = @glade[W_ENTRY_NEW_COLUMN].text
        log("new Column : "+name)
        column = createColumnMap(name)
        addColumn(getColumnSelectorModel(), column)
        changed()
      }
    end
    
    # clic on Delete Column button
    def on_buttonDeleteColumn_clicked(pWidget)
      log("delete column") {
        selectedRow = getColumnSelectorView().selection.selected
        return unless selectedRow
        column = selectedRow[0]
        if @application.configManager.isColumnDeletable(column.key)
          getColumnSelectorModel().remove(selectedRow)
          changed()
        else
          setError("Delete", "Deleting "+column.to_s+" is forbidden")
        end
      }
    end
    
        # clic on Delete Column button
    def on_checkEditable_toggled(pWidget)
      log("toggle editable")
      selectedRow = getColumnSelectorView().selection.selected
      return unless selectedRow
      columnMap = selectedRow[0]
      columnMap.editable = @glade[W_CHECK_EDITABLE].active?
    end

  end
end

puts "-- ColumnSelectorGUI"
